import _keys from 'lodash/keys';
import _omit from 'lodash/omit';
import _isEmpty from 'lodash/isEmpty';
import _isNil from 'lodash/isNil';
import _get from 'lodash/get';
import _set from 'lodash/set';
import _has from 'lodash/has';
import _pick from 'lodash/pick';
/**
 * @description 将树平铺为数组
 * @param { object } tree 数据
 * @param { String } children 树形结构关联的属性
 */
const getTreeMap = (tree, children = 'children') => {
  if (!(tree instanceof Array)) {
    return;
  }

  const treeMap = [];

  tree.forEach(node => {
    treeMap.push(...getNodeMap(node, tree, children));
  });

  return treeMap;
};

const getNodeMap = (node, parentNode, children) => {
  //   node.parentNode = JSON.parse(JSON.stringify(parentNode));
  const nodeMap = [node];

  if (node[children] && node[children].length) {
    node[children].forEach(item =>
      nodeMap.push(...getNodeMap(item, node, children))
    );
  }

  return nodeMap;
};

/**
 * @description 将数组变成tree非递归
 * @param {*} arr
 * @param {*} nodeKey 节点标识
 * @param {*} parentKey 父节点标识
 * @param {*} children 子节点属性
 * @returns
 */
const arrayToTree = (
  arr,
  nodeKey = 'id',
  parentKey = 'parentId',
  children = 'children'
) => {
  const result = [];

  if (!Array.isArray(arr) || arr.length === 0) {
    return result;
  }

  const map = {};

  arr.forEach(item => (map[item[nodeKey]] = item));
  arr.forEach(item => {
    const parent = map[item[parentKey]];

    if (parent) {
      (parent[children] || (parent[children] = [])).push(item);
      return;
    }

    result.push(item);
  });
  return result;
};

/**
 * @description 查找节点在树中的路径
 * @param {*} tree 树的数据
 * @param {*} id 查找的id
 * @param {*} nodeKey 节点标识符
 * @param {*} children 子节点标识
 * @returns
 */
const getNodePath = ({
  tree,
  id,
  nodeKey = 'id',
  children = 'children',
  dataAll = false,
  needKey
}) => {
  if (!Array.isArray(tree) || tree.length === 0) {
    return [];
  }

  const path = [];

  const treeFindPath = (childTree, childId, childPath) => {
    // eslint-disable-next-line no-restricted-syntax
    for (const item of childTree) {
      let itemData = null;
      if (needKey) {
        itemData = dataAll ? _omit(item, children) : item[needKey];
      } else {
        itemData = dataAll ? _omit(item, children) : item[nodeKey];
      }
      childPath.push(itemData);

      if (item[nodeKey] === childId) {
        return childPath;
      }

      if (item[children]) {
        const findChildren = treeFindPath(item[children], childId, childPath);

        if (findChildren.length) {
          return findChildren;
        }
      }

      childPath.pop();
    }

    return [];
  };

  return treeFindPath(tree, id, path);
};

/**
 * @description 模糊匹配树
 * @param {*} arr 树数据
 * @param {*} value  匹配的数据
 * @param {*} nameKey 查询的属性
 * @param {*} children 子节点属性
 * @returns
 */
const fuzzyQueryTree = (
  arr,
  value,
  nameKey = 'name',
  children = 'children'
) => {
  if (!Array.isArray(arr) || arr.length === 0) {
    return [];
  }

  const result = [];

  arr.forEach(item => {
    if (item[nameKey].indexOf(value) > -1) {
      const childrenNode = fuzzyQueryTree(
        item[children],
        value,
        nameKey,
        children
      );
      const obj = { ...item, childrenNode };

      result.push(obj);
      return;
    }

    if (item[children] && item[children].length > 0) {
      const childrenNode = fuzzyQueryTree(
        item[children],
        value,
        nameKey,
        children
      );
      const obj = { ...item, childrenNode };

      if (childrenNode && childrenNode.length > 0) {
        result.push(obj);
      }
    }
  });

  return result;
};

/**
 * @description 对树节点的属性进行操作
 * @param {Array} tree 树节点数据
 * @param { Function } callBack 回调函数
 * @param { string } children 子节点
 * @returns
 */
const operationAttrToNodes = (tree, callback, children = 'children') => {
  tree.forEach(item => {
    callback(item);

    if (item[children] && item[children].length > 0) {
      operationAttrToNodes(item[children], callback, children);
    }
  });

  return tree;
};

/**
 * @description 获取所有的叶子节点
 * @param {*} tree
 * @param {*} children
 * @returns
 */
const getAllLeaf = (tree, children = 'children') => {
  if (!Array.isArray(tree) || tree.length === 0) {
    return [];
  }

  const result = [];
  const getLeaf = treeLeaf => {
    treeLeaf.forEach(item => {
      if (!item[children] || item[children].length === 0) {
        result.push(item);
        return;
      }

      if (item[children] && item[children].length > 0) {
        getLeaf(item[children]);
      }
    });
  };

  getLeaf(tree);
  return result;
};
// 通过参数获取tree中的某一个节点
const getTreeNode = function(
  treeData = {},
  codeVal = '',
  codeLabel = 'code',
  children = 'children'
) {
  const code = _get(treeData, codeLabel);
  if (code === codeVal || code + '' === codeVal + '') {
    _set(
      treeData,
      `${children}Total`,
      _isNil(treeData[children]) ? 0 : treeData[children].length
    );
    return _pick(treeData, _keys(_omit(treeData, children)));
  }
  if (_has(treeData, children) && !_isEmpty(treeData[children])) {
    for (let i = 0, len = treeData[children].length; i < len; i++) {
      const obj = getTreeNode(
        treeData[children][i],
        codeVal,
        codeLabel,
        children
      );
      if (!_isNil(obj)) {
        return obj;
      }
    }
  }
};
// 通过参数获取tree中的某一个节点
const getArrayTreeNode = function(
  treeData = [],
  codeVal = '',
  codeLabel = 'code',
  children = 'children'
) {
  for (let i = 0, len = treeData.length; i < len; i++) {
    const obj = getTreeNode(treeData[i], codeVal, codeLabel, children);
    if (!_isNil(obj)) {
      return obj;
    }
  }
};
export {
  arrayToTree,
  getTreeMap,
  getNodePath,
  fuzzyQueryTree,
  operationAttrToNodes,
  getAllLeaf,
  getTreeNode,
  getArrayTreeNode
};
